@@ -3,11 +3,11 @@
 /* The name of this package (not localized) */
 const PACKAGE_NAME = 'gnome-shell';
 /* The version of this package */
-const PACKAGE_VERSION = '3.8.3';
+const PACKAGE_VERSION = '3.10.4';
 /* 1 if gnome-bluetooth is available, 0 otherwise */
 const HAVE_BLUETOOTH = 0;
-/* 1 if Network Manager is available, 0 otherwise */
-const HAVE_NETWORK_MANAGER = 0;
+/* 1 if networkmanager is available, 0 otherwise */
+const HAVE_NETWORKMANAGER = 0;
 /* gettext package */
 const GETTEXT_PACKAGE = 'gnome-shell';
 /* locale dir */
