@@ -15,12 +15,14 @@ const Signals = imports.signals;
 const Atk = imports.gi.Atk;
 
 
+const Animation = imports.ui.animation;
 const Config = imports.misc.config;
 const CtrlAltTab = imports.ui.ctrlAltTab;
 const DND = imports.ui.dnd;
 const Overview = imports.ui.overview;
 const PopupMenu = imports.ui.popupMenu;
 const PanelMenu = imports.ui.panelMenu;
+const RemoteMenu = imports.ui.remoteMenu;
 const Main = imports.ui.main;
 const Tweener = imports.ui.tweener;
 
@@ -28,7 +30,6 @@ const PANEL_ICON_SIZE = 24;
 
 const BUTTON_DND_ACTIVATION_TIMEOUT = 250;
 
-const ANIMATED_ICON_UPDATE_TIMEOUT = 100;
 const SPINNER_ANIMATION_TIME = 0.2;
 
 // To make sure the panel corners blend nicely with the panel,
@@ -74,81 +75,6 @@ function _unpremultiply(color) {
                                blue: blue, alpha: color.alpha });
 };
 
-const Animation = new Lang.Class({
-    Name: 'Animation',
-
-    _init: function(filename, width, height, speed) {
-        this.actor = new St.Bin();
-        this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
-        this._speed = speed;
-
-        this._isLoaded = false;
-        this._isPlaying = false;
-        this._timeoutId = 0;
-        this._frame = 0;
-        this._animations = St.TextureCache.get_default().load_sliced_image (filename, width, height,
-                                                                            Lang.bind(this, this._animationsLoaded));
-        this.actor.set_child(this._animations);
-    },
-
-    play: function() {
-        if (this._isLoaded && this._timeoutId == 0) {
-            if (this._frame == 0)
-                this._showFrame(0);
-
-            this._timeoutId = Mainloop.timeout_add(this._speed, Lang.bind(this, this._update));
-        }
-
-        this._isPlaying = true;
-    },
-
-    stop: function() {
-        if (this._timeoutId > 0) {
-            Mainloop.source_remove(this._timeoutId);
-            this._timeoutId = 0;
-        }
-
-        this._isPlaying = false;
-    },
-
-    _showFrame: function(frame) {
-        let oldFrameActor = this._animations.get_child_at_index(this._frame);
-        if (oldFrameActor)
-            oldFrameActor.hide();
-
-        this._frame = (frame % this._animations.get_n_children());
-
-        let newFrameActor = this._animations.get_child_at_index(this._frame);
-        if (newFrameActor)
-            newFrameActor.show();
-    },
-
-    _update: function() {
-        this._showFrame(this._frame + 1);
-        return true;
-    },
-
-    _animationsLoaded: function() {
-        this._isLoaded = true;
-
-        if (this._isPlaying)
-            this.play();
-    },
-
-    _onDestroy: function() {
-        this.stop();
-    }
-});
-
-const AnimatedIcon = new Lang.Class({
-    Name: 'AnimatedIcon',
-    Extends: Animation,
-
-    _init: function(filename, size) {
-        this.parent(filename, size, size, ANIMATED_ICON_UPDATE_TIMEOUT);
-    }
-});
-
 const TextShadower = new Lang.Class({
     Name: 'TextShadower',
 
@@ -263,7 +189,6 @@ const AppMenuButton = new Lang.Class({
 
         this.actor.bind_property("reactive", this.actor, "can-focus", 0);
         this.actor.reactive = false;
-        this._targetIsCurrent = false;
 
         this._container = new Shell.GenericContainer();
         bin.set_child(this._container);
@@ -281,20 +206,23 @@ const AppMenuButton = new Lang.Class({
         this._iconBox.connect('notify::allocation',
                               Lang.bind(this, this._updateIconBoxClip));
         this._container.add_actor(this._iconBox);
+
+        this._hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
+        this._container.add_actor(this._hbox);
+
         this._label = new TextShadower();
-        this._container.add_actor(this._label.actor);
+        this._label.actor.y_align = Clutter.ActorAlign.CENTER;
+        this._hbox.add_actor(this._label.actor);
+        this._arrow = PopupMenu.unicodeArrow(St.Side.BOTTOM);
+        this._hbox.add_actor(this._arrow);
 
         this._iconBottomClip = 0;
 
         this._visible = !Main.overview.visible;
         if (!this._visible)
             this.actor.hide();
-        this._overviewHidingId = Main.overview.connect('hiding', Lang.bind(this, function () {
-            this.show();
-        }));
-        this._overviewShowingId = Main.overview.connect('showing', Lang.bind(this, function () {
-            this.hide();
-        }));
+        this._overviewHidingId = Main.overview.connect('hiding', Lang.bind(this, this._sync));
+        this._overviewShowingId = Main.overview.connect('showing', Lang.bind(this, this._sync));
 
         this._stop = true;
 
@@ -317,13 +245,8 @@ const AppMenuButton = new Lang.Class({
             return;
 
         this._visible = true;
-        this.actor.show();
-
-        if (!this._targetIsCurrent)
-            return;
-
         this.actor.reactive = true;
-
+        this.actor.show();
         Tweener.removeTweens(this.actor);
         Tweener.addTween(this.actor,
                          { opacity: 255,
@@ -337,11 +260,6 @@ const AppMenuButton = new Lang.Class({
 
         this._visible = false;
         this.actor.reactive = false;
-        if (!this._targetIsCurrent) {
-            this.actor.hide();
-            return;
-        }
-
         Tweener.removeTweens(this.actor);
         Tweener.addTween(this.actor,
                          { opacity: 0,
@@ -359,10 +277,9 @@ const AppMenuButton = new Lang.Class({
         if (!success || this._spinnerIcon == icon)
             return;
         this._spinnerIcon = icon;
-        this._spinner = new AnimatedIcon(this._spinnerIcon, PANEL_ICON_SIZE);
-        this._container.add_actor(this._spinner.actor);
+        this._spinner = new Animation.AnimatedIcon(this._spinnerIcon, PANEL_ICON_SIZE);
+        this._hbox.add_actor(this._spinner.actor);
         this._spinner.actor.hide();
-        this._spinner.actor.lower_bottom();
     },
 
     _onIconBoxStyleChanged: function() {
@@ -371,13 +288,19 @@ const AppMenuButton = new Lang.Class({
         this._updateIconBoxClip();
     },
 
+    _syncIcon: function() {
+        if (!this._targetApp)
+            return;
+
+        let icon = this._targetApp.get_faded_icon(2 * PANEL_ICON_SIZE, this._iconBox.text_direction);
+        this._iconBox.set_child(icon);
+    },
+
     _onIconThemeChanged: function() {
         if (this._iconBox.child == null)
             return;
 
-        this._iconBox.child.destroy();
-        let icon = this._targetApp.get_faded_icon(2 * PANEL_ICON_SIZE);
-        this._iconBox.set_child(icon);
+        this._syncIcon();
     },
 
     _updateIconBoxClip: function() {
@@ -395,7 +318,6 @@ const AppMenuButton = new Lang.Class({
             return;
 
         this._stop = true;
-        this.actor.reactive = true;
 
         if (this._spinner == null)
             return;
@@ -415,7 +337,6 @@ const AppMenuButton = new Lang.Class({
 
     startAnimation: function() {
         this._stop = false;
-        this.actor.reactive = false;
 
         if (this._spinner == null)
             return;
@@ -428,7 +349,7 @@ const AppMenuButton = new Lang.Class({
         let [minSize, naturalSize] = this._iconBox.get_preferred_width(forHeight);
         alloc.min_size = minSize;
         alloc.natural_size = naturalSize;
-        [minSize, naturalSize] = this._label.actor.get_preferred_width(forHeight);
+        [minSize, naturalSize] = this._hbox.get_preferred_width(forHeight);
         alloc.min_size = alloc.min_size + Math.max(0, minSize - Math.floor(alloc.min_size / 2));
         alloc.natural_size = alloc.natural_size + Math.max(0, naturalSize - Math.floor(alloc.natural_size / 2));
     },
@@ -437,7 +358,7 @@ const AppMenuButton = new Lang.Class({
         let [minSize, naturalSize] = this._iconBox.get_preferred_height(forWidth);
         alloc.min_size = minSize;
         alloc.natural_size = naturalSize;
-        [minSize, naturalSize] = this._label.actor.get_preferred_height(forWidth);
+        [minSize, naturalSize] = this._hbox.get_preferred_height(forWidth);
         if (minSize > alloc.min_size)
             alloc.min_size = minSize;
         if (naturalSize > alloc.natural_size)
@@ -467,11 +388,10 @@ const AppMenuButton = new Lang.Class({
 
         let iconWidth = childBox.x2 - childBox.x1;
 
-        [minWidth, minHeight, naturalWidth, naturalHeight] = this._label.actor.get_preferred_size();
+        [minWidth, naturalWidth] = this._hbox.get_preferred_width(-1);
 
-        yPadding = Math.floor(Math.max(0, allocHeight - naturalHeight) / 2);
-        childBox.y1 = yPadding;
-        childBox.y2 = childBox.y1 + Math.min(naturalHeight, allocHeight);
+        childBox.y1 = 0;
+        childBox.y2 = allocHeight;
 
         if (direction == Clutter.TextDirection.LTR) {
             childBox.x1 = Math.floor(iconWidth / 2);
@@ -480,24 +400,7 @@ const AppMenuButton = new Lang.Class({
             childBox.x2 = allocWidth - Math.floor(iconWidth / 2);
             childBox.x1 = Math.max(0, childBox.x2 - naturalWidth);
         }
-        this._label.actor.allocate(childBox, flags);
-
-        if (this._spinner == null)
-            return;
-
-        if (direction == Clutter.TextDirection.LTR) {
-            childBox.x1 = Math.floor(iconWidth / 2) + this._label.actor.width;
-            childBox.x2 = childBox.x1 + this._spinner.actor.width;
-            childBox.y1 = box.y1;
-            childBox.y2 = box.y2 - 1;
-            this._spinner.actor.allocate(childBox, flags);
-        } else {
-            childBox.x1 = -this._spinner.actor.width;
-            childBox.x2 = childBox.x1 + this._spinner.actor.width;
-            childBox.y1 = box.y1;
-            childBox.y2 = box.y2 - 1;
-            this._spinner.actor.allocate(childBox, flags);
-        }
+        this._hbox.allocate(childBox, flags);
     },
 
     _onAppStateChanged: function(appSys, app) {
@@ -523,110 +426,88 @@ const AppMenuButton = new Lang.Class({
             // If the app has just lost focus to the panel, pretend
             // nothing happened; otherwise you can't keynav to the
             // app menu.
-            if (global.stage_input_mode == Shell.StageInputMode.FOCUSED)
+            if (global.stage.key_focus != null)
                 return;
         }
         this._sync();
     },
 
-    _sync: function() {
+    _findTargetApp: function() {
+        let workspace = global.screen.get_active_workspace();
         let tracker = Shell.WindowTracker.get_default();
         let focusedApp = tracker.focus_app;
-        let lastStartedApp = null;
-        let workspace = global.screen.get_active_workspace();
+        if (focusedApp && focusedApp.is_on_workspace(workspace))
+            return focusedApp;
+
         for (let i = 0; i < this._startingApps.length; i++)
             if (this._startingApps[i].is_on_workspace(workspace))
-                lastStartedApp = this._startingApps[i];
+                return this._startingApps[i];
 
-        let targetApp = focusedApp != null ? focusedApp : lastStartedApp;
+        return null;
+    },
 
-        if (targetApp == null) {
-            if (!this._targetIsCurrent)
-                return;
+    _sync: function() {
+        let targetApp = this._findTargetApp();
 
-            this.actor.reactive = false;
-            this._targetIsCurrent = false;
+        if (this._targetApp != targetApp) {
+            if (this._appMenuNotifyId) {
+                this._targetApp.disconnect(this._appMenuNotifyId);
+                this._appMenuNotifyId = 0;
+            }
+            if (this._actionGroupNotifyId) {
+                this._targetApp.disconnect(this._actionGroupNotifyId);
+                this._actionGroupNotifyId = 0;
+            }
 
-            Tweener.removeTweens(this.actor);
-            Tweener.addTween(this.actor, { opacity: 0,
-                                           time: Overview.ANIMATION_TIME,
-                                           transition: 'easeOutQuad' });
-            return;
-        }
+            this._targetApp = targetApp;
 
-        if (!targetApp.is_on_workspace(workspace))
-            return;
-
-        if (!this._targetIsCurrent) {
-            this.actor.reactive = true;
-            this._targetIsCurrent = true;
-
-            Tweener.removeTweens(this.actor);
-            Tweener.addTween(this.actor, { opacity: 255,
-                                           time: Overview.ANIMATION_TIME,
-                                           transition: 'easeOutQuad' });
-        }
-
-        if (targetApp == this._targetApp) {
-            if (targetApp && targetApp.get_state() != Shell.AppState.STARTING) {
-                this.stopAnimation();
-                this._maybeSetMenu();
+            if (this._targetApp) {
+                this._appMenuNotifyId = this._targetApp.connect('notify::menu', Lang.bind(this, this._sync));
+                this._actionGroupNotifyId = this._targetApp.connect('notify::action-group', Lang.bind(this, this._sync));
+                this._label.setText(this._targetApp.get_name());
+                this.actor.set_accessible_name(this._targetApp.get_name());
             }
-            return;
         }
 
-        if (this._spinner)
-            this._spinner.actor.hide();
-        if (this._iconBox.child != null)
-            this._iconBox.child.destroy();
-        this._iconBox.hide();
-        this._label.setText('');
+        let visible = (this._targetApp != null && !Main.overview.visibleTarget);
+        if (visible)
+            this.show();
+        else
+            this.hide();
 
-        if (this._appMenuNotifyId)
-            this._targetApp.disconnect(this._appMenuNotifyId);
-        if (this._actionGroupNotifyId)
-            this._targetApp.disconnect(this._actionGroupNotifyId);
-        if (targetApp) {
-            this._appMenuNotifyId = targetApp.connect('notify::menu', Lang.bind(this, this._sync));
-            this._actionGroupNotifyId = targetApp.connect('notify::action-group', Lang.bind(this, this._sync));
-        } else {
-            this._appMenuNotifyId = 0;
-            this._actionGroupNotifyId = 0;
-        }
-
-        this._targetApp = targetApp;
-        let icon = targetApp.get_faded_icon(2 * PANEL_ICON_SIZE);
-
-        this._label.setText(targetApp.get_name());
-        this.setName(targetApp.get_name());
-
-        this._iconBox.set_child(icon);
-        this._iconBox.show();
-
-        if (targetApp.get_state() == Shell.AppState.STARTING)
+        let isBusy = (this._targetApp != null &&
+                      (this._targetApp.get_state() == Shell.AppState.STARTING ||
+                       this._targetApp.get_state() == Shell.AppState.BUSY));
+        if (isBusy)
             this.startAnimation();
         else
-            this._maybeSetMenu();
+            this.stopAnimation();
 
+        this.actor.reactive = (visible && !isBusy);
+
+        this._syncIcon();
+        this._maybeSetMenu();
         this.emit('changed');
     },
 
     _maybeSetMenu: function() {
         let menu;
 
-        if (this._targetApp.action_group && this._targetApp.menu) {
-            if (this.menu instanceof PopupMenu.RemoteMenu &&
+        if (this._targetApp == null) {
+            menu = null;
+        } else if (this._targetApp.action_group && this._targetApp.menu) {
+            if (this.menu instanceof RemoteMenu.RemoteMenu &&
                 this.menu.actionGroup == this._targetApp.action_group)
                 return;
 
-            menu = new PopupMenu.RemoteMenu(this.actor, this._targetApp.menu, this._targetApp.action_group);
+            menu = new RemoteMenu.RemoteMenu(this.actor, this._targetApp.menu, this._targetApp.action_group);
             menu.connect('activate', Lang.bind(this, function() {
                 let win = this._targetApp.get_windows()[0];
                 win.check_alive(global.get_current_time());
             }));
 
         } else {
-            if (this.menu.isDummyQuitMenu)
+            if (this.menu && this.menu.isDummyQuitMenu)
                 return;
 
             // fallback to older menu
@@ -638,7 +519,8 @@ const AppMenuButton = new Lang.Class({
         }
 
         this.setMenu(menu);
-        this._menuManager.addMenu(menu);
+        if (menu)
+            this._menuManager.addMenu(menu);
     },
 
     destroy: function() {
@@ -683,7 +565,8 @@ const ActivitiesButton = new Lang.Class({
 
         /* Translators: If there is no suitable word for "Activities"
            in your language, you can use the word for "Overview". */
-        this._label = new St.Label({ text: _("Activities") });
+        this._label = new St.Label({ text: _("Activities"),
+                                     y_align: Clutter.ActorAlign.CENTER });
         this.actor.add_actor(this._label);
 
         this.actor.label_actor = this._label;
@@ -726,6 +609,7 @@ const ActivitiesButton = new Lang.Class({
 
     _onButtonRelease: function() {
         Main.overview.toggle();
+        this.menu.close();
     },
 
     _onKeyRelease: function(actor, event) {
@@ -914,33 +798,73 @@ const PanelCorner = new Lang.Class({
     }
 });
 
+const AggregateMenu = new Lang.Class({
+    Name: 'AggregateMenu',
+    Extends: PanelMenu.Button,
+
+    _init: function() {
+        this.parent(0.0, _("Settings"), false);
+        this.menu.actor.add_style_class_name('aggregate-menu');
+
+        this._indicators = new St.BoxLayout({ style_class: 'panel-status-indicators-box' });
+        this.actor.add_child(this._indicators);
+
+        if (Config.HAVE_NETWORKMANAGER) {
+            this._network = new imports.ui.status.network.NMApplet();
+        } else {
+            this._network = null;
+        }
+        if (Config.HAVE_BLUETOOTH) {
+            this._bluetooth = new imports.ui.status.bluetooth.Indicator();
+        } else {
+            this._bluetooth = null;
+        }
+
+        this._power = new imports.ui.status.power.Indicator();
+        this._rfkill = new imports.ui.status.rfkill.Indicator();
+        this._volume = new imports.ui.status.volume.Indicator();
+        this._brightness = new imports.ui.status.brightness.Indicator();
+        this._system = new imports.ui.status.system.Indicator();
+        this._screencast = new imports.ui.status.screencast.Indicator();
+
+        this._indicators.add_child(this._screencast.indicators);
+        if (this._network) {
+            this._indicators.add_child(this._network.indicators);
+        }
+        if (this._bluetooth) {
+            this._indicators.add_child(this._bluetooth.indicators);
+        }
+        this._indicators.add_child(this._rfkill.indicators);
+        this._indicators.add_child(this._volume.indicators);
+        this._indicators.add_child(this._power.indicators);
+        this._indicators.add_child(PopupMenu.unicodeArrow(St.Side.BOTTOM));
+
+        this.menu.addMenuItem(this._volume.menu);
+        this.menu.addMenuItem(this._brightness.menu);
+        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
+        if (this._network) {
+            this.menu.addMenuItem(this._network.menu);
+        }
+        if (this._bluetooth) {
+            this.menu.addMenuItem(this._bluetooth.menu);
+        }
+        this.menu.addMenuItem(this._rfkill.menu);
+        this.menu.addMenuItem(this._power.menu);
+        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
+        this.menu.addMenuItem(this._system.menu);
+    },
+});
+
 const PANEL_ITEM_IMPLEMENTATIONS = {
     'activities': ActivitiesButton,
+    'aggregateMenu': AggregateMenu,
     'appMenu': AppMenuButton,
     'dateMenu': imports.ui.dateMenu.DateMenuButton,
     'a11y': imports.ui.status.accessibility.ATIndicator,
     'a11yGreeter': imports.ui.status.accessibility.ATGreeterIndicator,
-    'volume': imports.ui.status.volume.Indicator,
-    'battery': imports.ui.status.power.Indicator,
-    'lockScreen': imports.ui.status.lockScreenMenu.Indicator,
     'keyboard': imports.ui.status.keyboard.InputSourceIndicator,
-    'powerMenu': imports.gdm.powerMenu.PowerMenuButton,
-    'userMenu': imports.ui.userMenu.UserMenuButton
 };
 
-if (Config.HAVE_BLUETOOTH)
-    PANEL_ITEM_IMPLEMENTATIONS['bluetooth'] =
-        imports.ui.status.bluetooth.Indicator;
-
-if (Config.HAVE_NETWORK_MANAGER) {
-    try {
-        PANEL_ITEM_IMPLEMENTATIONS['network'] =
-            imports.ui.status.network.NMApplet;
-    } catch(e) {
-        log('NMApplet is not supported. It is possible that your NetworkManager version is too old');
-    }
-}
-
 const Panel = new Lang.Class({
     Name: 'Panel',
 
