@@ -10,6 +10,8 @@ const FileUtils = imports.misc.fileUtils;
 const Main = imports.ui.main;
 const Params = imports.misc.params;
 
+const Config = imports.misc.config;
+
 const DEFAULT_MODE = 'restrictive';
 
 const _modes = {
@@ -21,6 +23,7 @@ const _modes = {
         showCalendarEvents: false,
         allowSettings: false,
         allowExtensions: false,
+        allowScreencast: false,
         enabledExtensions: [],
         hasRunDialog: false,
         hasWorkspaces: false,
@@ -48,8 +51,7 @@ const _modes = {
         panel: {
             left: [],
             center: ['dateMenu'],
-            right: ['a11yGreeter', 'display', 'keyboard',
-                    'volume', 'battery', 'powerMenu']
+            right: ['a11yGreeter', 'keyboard', 'aggregateMenu'],
         },
         panelStyle: 'login-screen'
     },
@@ -60,9 +62,9 @@ const _modes = {
         unlockDialog: undefined,
         components: ['polkitAgent', 'telepathyClient'],
         panel: {
-            left: ['userMenu'],
+            left: [],
             center: [],
-            right: ['lockScreen']
+            right: ['aggregateMenu']
         },
         panelStyle: 'lock-screen'
     },
@@ -72,29 +74,19 @@ const _modes = {
         unlockDialog: undefined,
         components: ['polkitAgent', 'telepathyClient'],
         panel: {
-            left: ['userMenu'],
+            left: [],
             center: [],
-            right: ['a11y', 'keyboard', 'lockScreen']
+            right: ['a11y', 'keyboard', 'aggregateMenu']
         },
         panelStyle: 'unlock-screen'
     },
 
-    'initial-setup': {
-        hasWindows: true,
-        isPrimary: true,
-        components: ['networkAgent', 'keyring'],
-        panel: {
-            left: [],
-            center: ['dateMenu'],
-            right: ['a11yGreeter', 'keyboard', 'volume', 'battery']
-        }
-    },
-
     'user': {
         hasOverview: true,
         showCalendarEvents: true,
         allowSettings: true,
         allowExtensions: true,
+        allowScreencast: true,
         hasRunDialog: true,
         hasWorkspaces: true,
         hasWindows: true,
@@ -102,13 +94,16 @@ const _modes = {
         isLocked: false,
         isPrimary: true,
         unlockDialog: imports.ui.unlockDialog.UnlockDialog,
-        components: ['polkitAgent', 'telepathyClient',
-                     'keyring', 'recorder', 'autorunManager', 'automountManager'],
+        components: Config.HAVE_NETWORKMANAGER ?
+                    ['networkAgent', 'polkitAgent', 'telepathyClient',
+                     'keyring', 'autorunManager', 'automountManager'] :
+                    ['polkitAgent', 'telepathyClient',
+                     'keyring', 'autorunManager', 'automountManager'],
+
         panel: {
             left: ['activities', 'appMenu'],
             center: ['dateMenu'],
-            right: ['a11y', 'keyboard', 'volume', 'bluetooth',
-                    'network', 'battery', 'userMenu']
+            right: ['a11y', 'keyboard', 'aggregateMenu']
         }
     }
 };
@@ -196,10 +191,6 @@ const SessionMode = new Lang.Class({
         return this._modeStack[this._modeStack.length - 1];
     },
 
-    get allowScreencast() {
-        return this.components.indexOf('recorder') != -1;
-    },
-
     _sync: function() {
         let params = this._modes[this.currentMode];
         let defaults;
